/**
 Copyright (c) 2016 Razeware LLC
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */

import UIKit

private class CheckerboardSquareCollectionViewCell: UICollectionViewCell {
    
    var squareString = "▪️" {
        didSet {
            switch squareString {
            case "🔴":
                checkerboardPieceView.backgroundColor = UIColor(red: 0.831372549, green: 0.235294118, blue: 0.129411765, alpha: 1.0)
            case "⚪️":
                checkerboardPieceView.backgroundColor = UIColor(red: 0.996078431, green: 0.980392157, blue: 0.960784314, alpha: 1.0)
            case _:
                checkerboardPieceView.backgroundColor = .clear
            }
        }
    }
    
    let checkerboardPieceView = UIView()
    
    override func didMoveToSuperview() {
        super.didMoveToSuperview()
        addSubview(checkerboardPieceView)
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        checkerboardPieceView.frame = bounds.insetBy(dx: 5.0, dy: 5.0)
        checkerboardPieceView.layer.cornerRadius = checkerboardPieceView.bounds.width / 2.0
    }
}

private let creamColor = UIColor(red: 0.992156863, green: 0.937254902, blue: 0.756862745, alpha: 1.0)
private let greenColor = UIColor(red: 0.376470588, green: 0.529411765, blue: 0.203921569, alpha: 1.0)

class CheckerboardView: UIView {
    
    fileprivate let collectionView = UICollectionView(frame: CGRect(), collectionViewLayout: UICollectionViewFlowLayout())
    fileprivate var checkerboardString: String
    
    override func layoutSubviews() {
        super.layoutSubviews()
        collectionView.frame = bounds.insetBy(dx: 5.0, dy: 5.0)
        if let flowLayout = collectionView.collectionViewLayout as? UICollectionViewFlowLayout {
            flowLayout.minimumLineSpacing = 0.0
            flowLayout.minimumInteritemSpacing = 0.0
            flowLayout.itemSize = CGSize(width: collectionView.frame.width / 8.0, height: collectionView.frame.height / 8.0)
        }
    }
    
    override var intrinsicContentSize: CGSize {
        return CGSize(width: 250.0, height: 250.0)
    }
    
    init(checkerboardString: String) {
        self.checkerboardString = checkerboardString
        super.init(frame: CGRect(x: 0.0, y: 0.0, width: 250.0, height: 250.0))
        collectionView.register(CheckerboardSquareCollectionViewCell.self, forCellWithReuseIdentifier: "Cell")
        addSubview(collectionView)
        backgroundColor = .brown
        collectionView.dataSource = self
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}

extension CheckerboardView: UICollectionViewDataSource {
    
    func numberOfSections(in collectionView: UICollectionView) -> Int {
        return 8
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return 8
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "Cell", for: indexPath) as! CheckerboardSquareCollectionViewCell
        
        if (indexPath.section % 2 == 0) {
            cell.contentView.backgroundColor = (indexPath.row % 2 == 0) ? creamColor : greenColor
        } else {
            cell.contentView.backgroundColor = (indexPath.row % 2 == 1) ? creamColor : greenColor
        }
        
        let squares = checkerboardString.components(separatedBy: "\n").map({ $0.characters.map({ String($0) })})
        cell.squareString = squares[indexPath.section][indexPath.row]
        return cell
    }
}
